// Compile this with:
// g++ -O3 -std=c++0x -pthread test.cc -o test

#include <sys/time.h>
#include "queues.h"
#include <iostream>
#include <thread>

struct Data {
  Data(int x, int y) : a(x), b(y) { }
  Data() : a(-1), b(-1) { }
  int a;
  int b;
};

void producer_func(FixedSizeQueueInterface<Data>* queue, int total_pushes) {
  int num_pushes = 0;
  while (num_pushes < total_pushes) {
    if (queue->Write(Data(12, 13))) {
      num_pushes++;
    }
  }
}

void consumer_func(FixedSizeQueueInterface<Data>* queue, int total_pulls) {
  int num_pulls = 0;
  while (num_pulls < total_pulls) {
    Data data;
    if (queue->Read(&data)) {
      num_pulls++;
    }
  }
}

int main() {
  const int kNumOperations = 10000000;
  const int kMaxQueueSize = 1000;

  struct timeval start;
  struct timeval end;

  std::cout << "Starting Mutex Queue\n";
  MutexFixedSizeQueue<Data> mutex_queue(kMaxQueueSize);
  gettimeofday(&start, NULL);
  std::thread producer_thread(producer_func, &mutex_queue, kNumOperations);
  std::thread consumer_thread(consumer_func, &mutex_queue, kNumOperations);
  producer_thread.join();
  consumer_thread.join();
  gettimeofday(&end, NULL);
  double elapsed_seconds = end.tv_sec - start.tv_sec + (end.tv_usec - start.tv_usec)/1e6;

  std::cout << "Mutex Queue:  " << kNumOperations/elapsed_seconds << " ops/sec\n";

  std::cout << "\nStarting LockFree Queue\n";

  SingleProducerSingleConsumerFixedSizeQueue<Data> lockfree_queue(kMaxQueueSize);
  gettimeofday(&start, NULL);
  std::thread producer_thread2(producer_func, &lockfree_queue, kNumOperations);
  std::thread consumer_thread2(consumer_func, &lockfree_queue, kNumOperations);
  producer_thread2.join();
  consumer_thread2.join();
  gettimeofday(&end, NULL);

  elapsed_seconds = end.tv_sec - start.tv_sec + (end.tv_usec - start.tv_usec)/1e6;

  std::cout << "Lock Free Queue:  " << kNumOperations/elapsed_seconds << " ops/sec\n";

  return 0;
}
